<?php
/*
Plugin Name: PWA Installer
Plugin URI: https://jejakkreasi.com
Description: Menambahkan fitur shortcut PWA dengan pengaturan plugin di WordPress.
Version: 1.0.2
Author: Bungrahman
*/

// Buat menu pengaturan di admin
function pwa_plugin_menu() {
    add_options_page('PWA Installer', 'PWA Installer', 'manage_options', 'pwa-installer', 'pwa_plugin_settings_page');
}
add_action('admin_menu', 'pwa_plugin_menu');

// Simpan pengaturan plugin
function pwa_save_settings() {
    if (isset($_POST['pwa_save'])) {
        update_option('pwa_icon_192', esc_url($_POST['pwa_icon_192']));
        update_option('pwa_icon_512', esc_url($_POST['pwa_icon_512']));
        update_option('pwa_popup_delay', intval($_POST['pwa_popup_delay']));
        update_option('pwa_background_color', sanitize_hex_color($_POST['pwa_background_color']));
        update_option('pwa_theme_color', sanitize_hex_color($_POST['pwa_theme_color']));
        pwa_create_manifest();
    }
}
add_action('admin_init', 'pwa_save_settings');

// Tambahkan skrip media uploader
function pwa_enqueue_media_uploader($hook) {
    if ('settings_page_pwa-installer' === $hook) {
        wp_enqueue_media();
        wp_enqueue_script('pwa-media-upload', plugin_dir_url(__FILE__) . 'media-upload.js', array('jquery'), null, true);
    }
}
add_action('admin_enqueue_scripts', 'pwa_enqueue_media_uploader');

// Halaman pengaturan plugin
function pwa_plugin_settings_page() {
    ?>
    <div class="wrap">
        <h2>PWA Installer Settings</h2>
        <form method="post" action="">
            <?php settings_fields('pwa-settings-group'); ?>
            <table class="form-table">
                <tr>
                    <th scope="row">Icon 192x192</th>
                    <td>
                        <input type="text" id="pwa_icon_192" name="pwa_icon_192" value="<?php echo get_option('pwa_icon_192'); ?>" />
                        <button type="button" class="button pwa-upload-button" data-target="#pwa_icon_192">Upload</button>
                    </td>
                </tr>
                <tr>
                    <th scope="row">Icon 512x512</th>
                    <td>
                        <input type="text" id="pwa_icon_512" name="pwa_icon_512" value="<?php echo get_option('pwa_icon_512'); ?>" />
                        <button type="button" class="button pwa-upload-button" data-target="#pwa_icon_512">Upload</button>
                    </td>
                </tr>
                <tr>
                    <th scope="row">Popup Delay (ms)</th>
                    <td><input type="number" name="pwa_popup_delay" value="<?php echo get_option('pwa_popup_delay', 5000); ?>" /></td>
                </tr>
                <tr>
                    <th scope="row">Background Color</th>
                    <td><input type="text" name="pwa_background_color" value="<?php echo get_option('pwa_background_color', '#ffffff'); ?>" /></td>
                </tr>
                <tr>
                    <th scope="row">Theme Color</th>
                    <td><input type="text" name="pwa_theme_color" value="<?php echo get_option('pwa_theme_color', '#007bff'); ?>" /></td>
                </tr>
            </table>
            <p class="submit"><input type="submit" name="pwa_save" class="button-primary" value="Save Settings" /></p>
        </form>
    </div>
    
    <style>
        #install-popup {
            position: fixed;
            bottom: 20px;
            left: 50%;
            transform: translateX(-50%);
            background: #007bff;
            color: white;
            padding: 15px;
            border-radius: 8px;
            display: none;
            max-width: 90%;
            width: 320px;
            text-align: center;
            box-shadow: 0px 4px 6px rgba(0, 0, 0, 0.1);
        }
        
        #install-popup button {
            margin: 10px 5px 0;
            padding: 8px 12px;
            background: white;
            color: #007bff;
            border: none;
            border-radius: 5px;
            cursor: pointer;
        }
        
        @media (max-width: 768px) {
            #install-popup {
                bottom: 10px;
                padding: 10px;
                width: 90%;
            }
        }
    </style>
    
    <div id="install-popup">
        <span>Pasang aplikasi untuk pengalaman lebih baik?</span>
        <button onclick="installPWA()">Pasang</button>
        <button onclick="closePopup()">Tutup</button>
    </div>
    
    <?php
}

// Tambahkan file JavaScript untuk media uploader
file_put_contents(plugin_dir_path(__FILE__) . 'media-upload.js', "
jQuery(document).ready(function($) {
    $('.pwa-upload-button').click(function(e) {
        e.preventDefault();
        var button = $(this);
        var target = $(button.data('target'));
        var file_frame = wp.media.frames.file_frame = wp.media({
            title: 'Pilih atau Upload Icon',
            button: { text: 'Gunakan Gambar' },
            multiple: false
        });
        file_frame.on('select', function() {
            var attachment = file_frame.state().get('selection').first().toJSON();
            target.val(attachment.url);
        });
        file_frame.open();
    });
});
");

function pwa_register_sw() {
    ?>
    <script>
        if ('serviceWorker' in navigator) {
            navigator.serviceWorker.register('/sw.js')
            .then(function(registration) {
                console.log('✅ Service Worker registered with scope:', registration.scope);
            })
            .catch(function(error) {
                console.error('❌ Service Worker registration failed:', error);
            });
        }
    </script>
    <?php
}
add_action('wp_footer', 'pwa_register_sw');

function pwa_create_sw_js() {
    $sw_file = ABSPATH . 'sw.js';

    // Ambil pengaturan dari database
    $icon_192 = get_option('pwa_icon_192', plugins_url('icon-192.png', __FILE__));
    $icon_512 = get_option('pwa_icon_512', plugins_url('icon-512.png', __FILE__));
    $background_color = get_option('pwa_background_color', '#ffffff');
    $theme_color = get_option('pwa_theme_color', '#007bff');
    $popup_delay = get_option('pwa_popup_delay', 5000);
    $theme_css = get_stylesheet_uri(); // CSS Tema
    $elementor_css = includes_url('css/dist/block-library/style.min.css'); // Elementor default CSS
    $elementor_global_css = includes_url('css/dist/block-library/theme.min.css'); // Elementor global styles


    // Isi file sw.js dengan pengaturan yang disimpan
    $sw_content = "
const CACHE_NAME = 'pwa-cache-v1';
const urlsToCache = [
    '/',
    '$icon_192',
    '$icon_512',
    '$theme_css',
    '$elementor_css',
    '$elementor_global_css'
];

self.addEventListener('install', event => {
    event.waitUntil(
        caches.open(CACHE_NAME).then(cache => {
            return cache.addAll(urlsToCache);
        })
    );
});

self.addEventListener('fetch', event => {
    event.respondWith(
        caches.match(event.request).then(response => {
            return response || fetch(event.request);
        })
    );
});

self.addEventListener('activate', event => {
    event.waitUntil(
        caches.keys().then(keys => {
            return Promise.all(
                keys.filter(key => key !== CACHE_NAME).map(key => caches.delete(key))
            );
        })
    );
});

// Menampilkan popup install dengan delay dari pengaturan
self.addEventListener('message', event => {
    if (event.data && event.data.type === 'SHOW_INSTALL_PROMPT') {
        setTimeout(() => {
            event.source.postMessage({ type: 'SHOW_POPUP' });
        }, $popup_delay);
    }
});
    ";

    // Buat file sw.js di root WordPress
    file_put_contents($sw_file, $sw_content);
}
register_activation_hook(__FILE__, 'pwa_create_sw_js');

// Buat & perbarui manifest.json
function pwa_create_manifest() {
    $manifest_file = ABSPATH . 'manifest.json';

    $manifest_data = [
        "name" => get_bloginfo('name'),
        "short_name" => get_bloginfo('name'),
        "start_url" => home_url('/'),
        "display" => "standalone",
        "background_color" => get_option('pwa_background_color', '#ffffff'),
        "theme_color" => get_option('pwa_theme_color', '#007bff'),
        "icons" => [
            [
                "src" => get_option('pwa_icon_192', plugins_url('icon-192.png', __FILE__)),
                "sizes" => "192x192",
                "type" => "image/png"
            ],
            [
                "src" => get_option('pwa_icon_512', plugins_url('icon-512.png', __FILE__)),
                "sizes" => "512x512",
                "type" => "image/png"
            ]
        ]
    ];

    file_put_contents($manifest_file, json_encode($manifest_data, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES));
}

// Buat manifest.json saat plugin diaktifkan
register_activation_hook(__FILE__, 'pwa_create_manifest');

function pwa_add_manifest_link() {
    echo '<link rel="manifest" href="' . home_url('/manifest.json') . '">';
}
add_action('wp_head', 'pwa_add_manifest_link');

